
import logging
import sys
from typing import List, Optional

import pkg_resources

from appbridge import bonjour, common, server, client
from appbridge.prototol import AppName, Msg, MsgType

# logging.basicConfig(level=logging.DEBUG)

FILES_FLAG = '--with-files'

def handle_reply(msg: Msg, received_files: List[str]):
    print('reply:', msg.toPrettyString())
    if msg.action == AppName.GetFiles:
        # move received file to files_dir
        common.save_files(received_files)
        pass
    else:
        # do nothing
        pass


def main():
    args = sys.argv[1:]
    cmd(args)

def cmd(args: List[str]):
    argslen = len(args)
    # no params
    if argslen == 0:
        version = pkg_resources.require("appbridge")[0].version
        print("appbridge v%s" % version)

    elif args[0] == 'discover':
        bonjour.discover_service()

    elif args[0] == 'start':
        # start <service_name> [<service_dir>] [-f <files_dir>]
        service_name = args[1]
        service_dir = ''
        files_dir = ''
        if argslen > 2 :
            flag = args[2]
            if flag == '-f':
                files_dir = args[3]
            else:
                service_dir = flag
                if argslen > 3 and args[3] == '-f':
                    files_dir = args[4]

        server.start_service(service_name, service_dir, files_dir)

    elif args[0] == 'request':
        # request <service_name> [ <app> [<params...>] [--with-files <files...>]]
        service_name = args[1]
        if argslen == 2:
            client.request_info(service_name)
            return

        action = args[2]
        try:
            dash_index = args.index(FILES_FLAG)
            params = args[3:dash_index]
            files = args[dash_index+1:]
        except:
            params = args[3:]
            files = []
        msg = Msg.request('appbridge', action, params, files)
        print('req:', msg.toPrettyString())
        client.request(service_name, msg, handle_reply)
    
    elif args[0] == 'send':
        newArgs = ['request', args[1], AppName.SendFiles]
        files = args[2:]
        newArgs.extend(files)
        newArgs.append(FILES_FLAG)
        newArgs.extend(files)
        # print(newArgs)
        cmd(newArgs)

    elif args[0] == 'get':
        newArgs = ['request', args[1], AppName.GetFiles]
        newArgs.extend(args[2:])
        # print(newArgs)
        cmd(newArgs)

    elif args[0] == 'ls':
        newArgs = ['request', args[1], AppName.ListFiles]
        newArgs.extend(args[2:])
        # print(newArgs)
        cmd(newArgs)

    else:
        print('action \"%s\" is not support!' % args[0])
        pass


if __name__ == '__main__':
    main()

