

from asyncio import sleep
from email.encoders import encode_quopri
from multiprocessing import Pipe, Process, Queue
from multiprocessing.connection import Connection
import shutil
# from time import sleep
from typing import AsyncGenerator, List
import os
from appbridge.cipher import Cipher
from appbridge.config import Config

from appbridge.prototol import DataType, Msg, MsgType, TransData
from appbridge.progress_bar import TranferBar











async def all_file_paths_generator(dir_or_file) -> AsyncGenerator[str, None]:
    # print("test %s" % dir_or_file)
    if os.path.isfile(dir_or_file):
        yield dir_or_file

    elif os.path.isdir(dir_or_file):
        for item in os.listdir(dir_or_file):
            async for p in all_file_paths_generator('%s/%s' % (dir_or_file, item)):
                yield p
    else:
        raise Exception("%s Not found!" % dir_or_file)

async def file_generator(abspath, cipher: Cipher) -> AsyncGenerator[bytes, None]:
        # the_path = os.path.join(dir, related_path) #'%s/%s/'%(dir, related_path) if dir != '' else related_path
        print("Will Generate files from \"%s\"" % abspath)
        basename = os.path.basename(abspath)
        if os.path.isdir(abspath):
            resp = Msg(MsgType.ReplyDirInfo, params=[basename])
            yield TransData.encrypted(resp, cipher).toBytes()

        # send new file cipher
        file_key = Cipher.random()
        file_iv = Cipher.random()
        msg = Msg(MsgType.ReplyFileCipher, params=[file_key.hex(), file_iv.hex()])
        yield TransData.encrypted(msg, cipher).toBytes()

        # start a process to read and encrypt files
        (c1, c2) = Pipe()
        fileReader = Process(target=read_file_process, args=(c1, file_key, file_iv))
        fileReader.start()

        # send every file
        async for file_path in all_file_paths_generator(abspath):
            
            print("File: %s" % file_path)
            # file_related_path = os.path.basename(filepath)
            file_size = os.path.getsize(file_path)
            # print("file size = %d" % file_size)
            # bar = TranferBar('Sending', max=file_size)

            if file_path == abspath:
                # single file
                # file_name = os.path.basename(related_path)
                resp = Msg(MsgType.ReplyFileInfo, params=[basename, file_size])
                yield TransData.encrypted(resp, cipher).toBytes()
            else:
                # file in dir
                file_name = os.path.relpath(file_path, abspath)
                # file_name = file_path.replace(the_path, '')
                resp = Msg(MsgType.ReplyFileInfo, params=[file_name, file_size])
                yield TransData.encrypted(resp, cipher).toBytes()

            # print("File: \"%s\"" % file_path)

            c2.send((file_path, file_size))
            while True:
                data = c2.recv_bytes()
                if data == b'':
                    break
                yield data

        # all files sent
        msg = Msg(MsgType.ReplyFileEnd)
        yield TransData.encrypted(msg, cipher).toBytes()

        # stop read process
        c2.send(('', 0))
        fileReader.join()

        print('end of read file!')
        

# def encrypt_process(read_queue: Queue, encrypt_queue: Queue, cipher, max=3):
#     # cipher = Cipher(key, iv)
#     # print('start encrypt process...') 
#     cnt = 0
#     while max > 0:
#         max -= 1
#         if read_queue.empty():
#             break

#         if encrypt_queue.full():
#             break

#         part = read_queue.get()
#         cnt += 1
#         # print('get part:', part)
#         if part == b'':
#             encrypt_queue.put(part)
#             break
#         encrypt_queue.put(TransData.encrypted_bin(part, cipher).toBytes())

#     return cnt
    

def read_file_process(pipe_c: Connection, key: bytes, iv: bytes):
    print('start read files process...')

    cipher = Cipher(key, iv)

    while True:
        file_path, file_size = pipe_c.recv()
        if file_path == '':
            break

        print("File: %s" % file_path)
        bar = TranferBar('Sending', max=file_size)

        with  open(file_path, 'rb') as file:
            while True:
                part = file.read(1024*1024)
                if len(part) <= 0:
                    pipe_c.send_bytes(b'')
                    break
                bar.next(len(part))
                pipe_c.send_bytes(TransData.encrypted_bin(part, cipher).toBytes())

        bar.finish()

    print('read files process end!!!')

def write_file_process(pipe: Connection, key: bytes, iv: bytes):
    print('start write files process...')

    cipher = Cipher(key, iv)

    while True:
        file_path, file_size = pipe.recv()
        if file_path == '':
            # process end
            break


        print("File: %s" % file_path)
        bar = TranferBar('Receiving', max=file_size)

        received_size = 0
        with open(file_path, 'wb') as file:
            while True:
                data = pipe.recv()
                # if data == b'':
                #     # one file end
                #     break
                # print('data to write[%d]:'%len(data), data)
                part = cipher.decrypt(data)
                file.write(part)
                partlen = len(part)
                bar.next(len(part))

                received_size += partlen
                if received_size >= file_size:
                    break

        bar.finish()

    print('read files process end!!!')


def start_file_writer_process(key, iv):
    (c1, c2) = Pipe()
    fileWriter = Process(target=write_file_process, args=(c1, key, iv))
    fileWriter.start()
    return (c2, fileWriter)

def save_files(received_files: List[str]):
    # print('save to files dir:  %s' % Config.files_dir)
    if Config.files_dir != '':
        for f in received_files:
            # print("File : %s" % f)

            # local_path = os.path.join(session_dir, f)
            filename = os.path.basename(f)
            path_on_files_dir = os.path.join(Config.files_dir, filename)

            if Config.rename_save:
                index = 1
                while os.path.exists(path_on_files_dir):
                    path_on_files_dir = os.path.join(Config.files_dir, str(index) + '_' + filename)
                    index += 1
            else:
                # delete exist dir
                if os.path.isdir(path_on_files_dir):
                    print("dir already exist, delete!")
                    shutil.rmtree(path_on_files_dir, ignore_errors=True)
            # print('from: ', f)
            shutil.move(f, path_on_files_dir)
            print("File saved in: %s" % path_on_files_dir)
