

import os
import shutil
from typing import Optional
import rsa

class RSAKey:

    prikey_filename = 'prikey.pem'
    pubkey_filename = 'pubkey.pem'

    prikey: Optional[any]
    pubkey: any


    def __init__(self, prikey: Optional[any], pubkey: any):
        self.pubkey = pubkey
        self.prikey = prikey
        # print('pub', pubkey)
        # print('pri', prikey)

    @classmethod
    def fromBytes(cls, pri_bytes: Optional[bytes], pub_bytes: bytes):
        if pri_bytes:
            prikey = rsa.PrivateKey.load_pkcs1(pri_bytes)
        else:
            prikey = None
        pubkey = rsa.PublicKey.load_pkcs1(pub_bytes)
        return RSAKey(prikey, pubkey)

    @classmethod
    def new(cls, dir):
        print('create new rsa keys')
        (pub, pri) = rsa.newkeys(2048)
        ins = RSAKey(pri, pub)
        ins.save(dir)
        return ins

    @classmethod
    def load(cls, dir):
        print('load rsa keys from %s' % dir)
        prikey_file = os.path.join(dir, RSAKey.prikey_filename)
        pubkey_file = os.path.join(dir, RSAKey.pubkey_filename)
        if os.path.isfile(prikey_file) and os.path.isfile(pubkey_file):
            with open(prikey_file, mode='rb') as privatefile:
                pri_keydata = privatefile.read()
                # prikey = rsa.PrivateKey.load_pkcs1(pri_keydata)
            with open(pubkey_file, mode='rb') as pubfile:
                pub_keydata = pubfile.read()
                # pubkey = rsa.PublicKey.load_pkcs1(pub_keydata)

            return RSAKey.fromBytes(pri_keydata, pub_keydata)

        else:
            return RSAKey.new(dir)

    def save(self, dir):
        print('save rsa keys to %s' % dir)
        pubkey_file = os.path.join(dir, RSAKey.pubkey_filename)
        os.makedirs(os.path.dirname(pubkey_file), 0o755, exist_ok=True)
        with open(pubkey_file, mode='wb+') as file:
            file.write(self.pubkey_bytes)

        prikey_file = os.path.join(dir, RSAKey.prikey_filename)
        os.makedirs(os.path.dirname(prikey_file), 0o755, exist_ok=True)
        with open(prikey_file, mode='wb+') as file:
            file.write(self.prikey_bytes)

    @property
    def pubkey_bytes(self) -> bytes:
        return self.pubkey.save_pkcs1()

    @property
    def prikey_bytes(self) -> bytes:
        return self.prikey.save_pkcs1()

    @property
    def pubkey_hex(self):
        return self.pubkey_bytes.hex()

    def encrypt(self, data: bytes) -> bytes:
        # print('data=', data)
        # print('pubkey', self.pubkey)
        return rsa.encrypt(data, self.pubkey)

    def encryptAsHex(self, data: bytes) -> str:
        return self.encrypt(data).hex()

    def decrypt(self, data: bytes) -> bytes:
        return rsa.decrypt(data, self.prikey)
