

from asyncio import Future
from glob import glob
from nis import cat
import os
import shutil
import traceback
from appbridge.rsa_key import RSAKey

from quart import Quart
from typing import Tuple, Optional
from rsocket.helpers import create_future
from rsocket.payload import Payload
from rsocket.request_handler import BaseRequestHandler
from rsocket.transports.quart_websocket import websocket_handler
from reactivestreams.publisher import Publisher
from reactivestreams.subscriber import Subscriber

from appbridge import bonjour, channel
from appbridge.prototol import AppName, ConnectMsg, DataType, MsgType, Msg, TransData
from appbridge.config import Config


class Handler(BaseRequestHandler):

    _rsa_key: Optional[RSAKey] = None
    
    @property
    def rsa_key(self):
        if self._rsa_key:
            return self._rsa_key
        else:
            self._rsa_key = RSAKey.load(Config.config_dir)
            return self._rsa_key

    async def request_response(self, payload: Payload) -> Future:
        # print(payload.data)
        msg = Msg.fromBytes(payload.data)

        # print('msg:', msg.toString())

        # get service information
        if msg.type == MsgType.GetInfo:
            # print('handle %s' % msg.type)
            reply = Msg(MsgType.Reply, params=[])
            if Config.config_dir != '' and os.path.isdir(Config.config_dir):
                for item in os.listdir(Config.config_dir):
                    if item.endswith('.py'):
                        reply.params.append(os.path.splitext(item)[0])
            if Config.files_dir != '':
                reply.params.append(AppName.GetFiles)
                reply.params.append(AppName.SendFiles)
                reply.params.append(AppName.ListFiles)
            # print(reply.toString())
            return create_future(Payload(reply.toBytes()))
        
        print('echo')
        # otherwise echo
        return create_future(payload)

    async def request_channel(self, payload: Payload) -> Tuple[Optional[Publisher], Optional[Subscriber]]:
        # print('payload', payload)
        data = TransData.fromBytes(payload.data)
        if data.type != DataType.connect:
            return None, None

        # msg = Msg.fromBytes(payload.data)
        msg: ConnectMsg = data.resolveMsg()
        # print('connect msg = ', msg)
        return channel.create(self.rsa_key, msg, is_service=True)
       
        
def start_service(name, dir, fdir):

    Config.config_dir = dir
    Config.files_dir = fdir

    port = 3000
    info = bonjour.register_service(name, port)
    name = info.name.split(".")[0]

    print("start service %s. service_dir=%s, files_dir=%s" % (name ,dir ,fdir)) 

    # session_dir = os.path.join(Config.service_dir, 'sessions') 
    # if os.path.isdir(session_dir):
    #     shutil.rmtree(session_dir, ignore_errors=True)

    if not os.path.exists(dir):
        print('Service dir not exit in %s' % dir)
        return
    
    if not os.path.exists(fdir):
        os.makedirs(fdir, 0o755)
    elif os.path.isfile(fdir):
        print('Service File dir should be a dir but it is a file! in %s' % fdir)
        return

    app = Quart(name)

    @app.websocket("/")
    async def ws():
        await websocket_handler(handler_factory=Handler)

    # logging.basicConfig(level=logging.ERROR)
    
    while port < 9999:
        try:
            app.run('0.0.0.0', port=port, use_reloader=False)
            print("app end!")
            break
        except KeyboardInterrupt:
            break
        except:
            print("failed to run on port %d" % (port))
            port+=1
            # bonjour.unregister_service(info)
            info.port = port
            bonjour.update_service(info)

    bonjour.unregister_service(info)
    
